@@ -20,8 +20,8 @@ except ImportError:
 
 from .perms import REDIRECT_FIELD_NAME, is_valid
 from .utils import (bcrypt, scrypt, split_hash, generate_salt, pbkdf2,
-    hash_sha256, hash_sha512, hash_bcrypt, hash_scrypt,
-    get_user_model, LazyUser, Token, auth_loader)
+    hash_sha256, hash_sha512, hash_bcrypt, hash_scrypt, get_user_hmac,
+    LazyUser, Token, auth_loader)
 
 
 __all__ = (
@@ -33,6 +33,7 @@ CREDENTIAL_LOGIN = 'login'
 CREDENTIAL_PASSWORD = 'password'
 CREDENTIAL_TOKEN = 'token'
 
+
 class UserExistsError(ValueError):
     
     def __init__(self, login):
@@ -74,7 +75,7 @@ class Auth(object):
     :   named settings. The following settings are available.
     
 
-    # Settings
+    ## Settings
     
     pepper
     :   required.
@@ -233,9 +234,8 @@ class Auth(object):
         if self.render:
             self.init_render()
     
-    
     def init_app(self, app):
-        """This callback can be used to initialize an application for the
+        """This method can be used to initialize an application for the
         use with this authentication setup.
         """
         self.app = app
@@ -246,7 +246,6 @@ class Auth(object):
         def set_user(request, **kwargs):
             self.get_user(request)
 
-
     def init_render(self):
         loader = ChoiceLoader([
             self.render.env.loader,
@@ -260,9 +259,15 @@ class Auth(object):
 
         self.auth_render.response_class = self.render.response_class
         self.auth_render.default_mimetype = self.render.default_mimetype
-    
 
-    def _get_user(self, login):
+    def render_to_string(self, template, context):
+        try:
+            return self.render(template, context, to_string=True)
+        except TemplateNotFound:
+            return auth_render(template, context, to_string=True)
+
+
+    def get_user_by_login(self, login):
         """Get user by login (case-insensitive).
         """
         login = login.lower()
@@ -270,10 +275,9 @@ class Auth(object):
         return self.db.query(self.User) \
             .filter(lower_(self.User.login) == login.lower()).first()
 
-
     def get_user(self, request):
-        request.__class__.user = self.LazyUser(self.db, self.User)
-    
+        request.__class__.user = self.LazyUser(self.app, self.db, self.User)
+
 
     def authenticate(self, credentials, update=True):
         """Check if there's a matching pair of login/hashed_password in the
@@ -295,10 +299,9 @@ class Auth(object):
         if token:
             token = to_bytestring(token)
             return self._token_authentication(token)
-    
 
     def _loginpass_authentication(self, login, password, update):
-        user = self._get_user(login)
+        user = self.get_user_by_login(login)
         if not user:
             return None
         if not self.check_password(password, user.password):
@@ -313,7 +316,6 @@ class Auth(object):
             if must_update_hash or must_update_cost:
                 user.password = self.hash_password(password)
         return user
-    
 
     def _token_authentication(self, token):
         try:
@@ -321,7 +323,7 @@ class Auth(object):
         except ValueError:
             return None
         
-        user = self._get_user(login)
+        user = self.get_user_by_login(login)
         if not user:
             return None
         
@@ -336,13 +338,11 @@ class Auth(object):
             return None
         
         return user
-    
 
     def hash_password(self, password):
         password += self.settings.pepper
         hashed = self.hash_func(password, cost=self.settings.hash_cost)
         return hashed
-    
 
     def check_password(self, password, stored_password):
         """Returns a boolean of whether the password is correct.
@@ -389,13 +389,11 @@ class Auth(object):
             raise ValueError("Hash algorithm '%s' not available" % hash_alg)
 
         return stored_password == hashed
-    
 
     def set_password(self, user, passw):
         if len(passw) < self.settings.password_minlen:
             raise PasswordTooShortError(self.settings.password_minlen)
         user.password = self.hash_password(passw)
-    
 
     def get_reset_token(self, user, time36=None):
         """Make a timestamped one-time-use token that can be used to
@@ -422,15 +420,15 @@ class Auth(object):
         mac = mac.hexdigest()[:50]
         token = '%s.%s.%s' % (user.login, time36, mac)
         return Token(token, self.settings.reset_expire)
-    
 
     def sign_in(self, request, user):
-        self.LazyUser.sign_in(request, user)
-    
+        request.session[self.LazyUser.SESSION_NAME] = get_user_hmac(user)
+        request.user = user
 
     def sign_out(self, request):
-        self.LazyUser.sign_out(request)
-    
+        self.app.session_interface.invalidate(request)
+        request.user = None
+
 
     def create_user(self, login, passw, **data):
         """[-login] USER_NAME [-passw] PASSWORD
@@ -439,7 +437,7 @@ class Auth(object):
         """
         login = to_bytestring(login)
         passw = to_bytestring(passw)
-        user_exists = self._get_user(login)
+        user_exists = self.get_user_by_login(login)
         if user_exists:
             raise UserExistsError(login)
         user = self.User(login=login, password='', **data)
@@ -448,7 +446,6 @@ class Auth(object):
         self.db.commit()
         return user
     
-    
     def change_password(self, login, passw):
         """[-login] USER_NAME [-passw] NEW_PASSWORD
         Changes the password of an existing user.
@@ -456,20 +453,13 @@ class Auth(object):
         """
         login = to_bytestring(login)
         passw = to_bytestring(passw)
-        user = self._get_user(login)
+        user = self.get_user_by_login(login)
         if not user:
             raise UserDoesNotExistsError(login)
         self.set_password(user, passw)
         self.db.commit()
         return user
-    
 
-    def render_to_string(self, template, context):
-        try:
-            return self.render(template, context, to_string=True)
-        except TemplateNotFound:
-            return auth_render(template, context, to_string=True)
-    
 
     def sign_in_view(self, request, **kwargs):
         """Signs a user in.
@@ -501,7 +491,6 @@ class Auth(object):
         
         template = self.settings.sign_in_template
         return self.auth_render(template, kwargs)
-    
 
     def sign_out_view(self, request, **kwargs):
         """Signs out a user.
@@ -519,7 +508,6 @@ class Auth(object):
             kwargs['auth'] = self
             return self.auth_render(template, kwargs)
         return redirect_to(redirect_url)
-    
 
     def reset_password_view(self, request, token=None, **kwargs):
         """A apge to generate a link to reset your password.
@@ -551,7 +539,7 @@ class Auth(object):
         # Phase 2
         elif request.is_post:
             login = request.form.get('login', '')
-            user = self._get_user(login)
+            user = self.get_user_by_login(login)
             if not user:
                 kwargs['error'] = 'NOT FOUND'
             else:
@@ -564,7 +552,6 @@ class Auth(object):
         
         template = self.settings.reset_password_template
         return self.auth_render(template, kwargs)
-    
 
     def email_token(self, user, token, site_name):
         assert self.mailer
@@ -588,7 +575,6 @@ class Auth(object):
             email_msg.text_content = msg
 
         self.mailer.send(email_msg)
-    
 
     def change_password_view(self, request, user_requested=True, **kwargs):
         """A view to change your password.
@@ -635,7 +621,6 @@ class Auth(object):
         template = self.settings.change_password_template
         return self.auth_render(template, kwargs)
 
-
     sign_in_controller = sign_in_view
     sign_out_controller = sign_out_view
     change_password_controller = change_password_view
